package test;

import java.io.Serializable;
import java.util.Collection;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.ejb.Timeout;
import javax.ejb.Timer;
import javax.ejb.TimerService;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;

@Stateless
public class MyTimerBean implements MyTimer {

    private static final Logger LOG = Logger.getLogger(Timer.class.getName());
	private static final long MS = 1000;

    @Resource
    private TimerService timerService;

    public void initTimers() {
        try {
            stopAllTimers();
			createTimer(MS, 0);
        } catch (Exception e) {
            LOG.log(Level.SEVERE, "Failed to init", e);
        }
    }

    @SuppressWarnings("unused")
    @Timeout
	@TransactionAttribute(TransactionAttributeType.NOT_SUPPORTED)
    private void timerTriggered(final Timer timer) {
        final Serializable info = timer.getInfo();
        if (info instanceof Long) {
            Long count = (Long) info;
            LOG.warning("Count " + count);
            createTimer(MS, count.longValue());
        }
    }

    private void createTimer(long nextCallMs, long count) {
        timerService.createTimer(nextCallMs, Long.valueOf(count + 1));
    }

    @SuppressWarnings("unchecked")
    private void stopAllTimers() {
        LOG.warning("Stopping all timers");
        for (final Timer timer : (Collection<Timer>) timerService.getTimers()) {
            timer.cancel();
        }
    }

}
